
package robotis.fluid;

import robotis.init.RobotisModItems;
import robotis.init.RobotisModFluids;
import robotis.init.RobotisModFluidTypes;
import robotis.init.RobotisModBlocks;

import net.minecraftforge.fluids.ForgeFlowingFluid;

import net.minecraft.world.level.material.FluidState;
import net.minecraft.world.level.material.Fluid;
import net.minecraft.world.level.block.state.StateDefinition;
import net.minecraft.world.level.block.LiquidBlock;
import net.minecraft.core.particles.ParticleTypes;
import net.minecraft.core.particles.ParticleOptions;

public abstract class SecureFluid extends ForgeFlowingFluid {
	public static final ForgeFlowingFluid.Properties PROPERTIES = new ForgeFlowingFluid.Properties(() -> RobotisModFluidTypes.SECURE_TYPE.get(),
			() -> RobotisModFluids.SECURE.get(), () -> RobotisModFluids.FLOWING_SECURE.get()).explosionResistance(100f).tickRate(1)
			.slopeFindDistance(16).bucket(() -> RobotisModItems.SECURE_BUCKET.get()).block(() -> (LiquidBlock) RobotisModBlocks.SECURE.get());

	private SecureFluid() {
		super(PROPERTIES);
	}

	@Override
	public ParticleOptions getDripParticle() {
		return ParticleTypes.FALLING_WATER;
	}

	public static class Source extends SecureFluid {
		public int getAmount(FluidState state) {
			return 8;
		}

		public boolean isSource(FluidState state) {
			return true;
		}
	}

	public static class Flowing extends SecureFluid {
		protected void createFluidStateDefinition(StateDefinition.Builder<Fluid, FluidState> builder) {
			super.createFluidStateDefinition(builder);
			builder.add(LEVEL);
		}

		public int getAmount(FluidState state) {
			return state.getValue(LEVEL);
		}

		public boolean isSource(FluidState state) {
			return false;
		}
	}
}
